import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import org.gjt.sp.jedit.*;
import org.gjt.sp.util.*;
import java.io.*;

class CopyAction extends Thread
{
    Component parent;
    File[] source;
    File destDir;
    CopyProgressDialog progressDialog = new CopyProgressDialog();
    boolean stop = false;
    
    private static final int COPY_IGNORE = -1;
    private static final int COPY_STOP = 0;
    private static final int COPY_OVERWRITE = 1;
    private static final int COPY_OVERWRITE_ALL = 2;
    private static final int COPY_SKIP = 3;
    
    int existsAction = CopyAction.COPY_IGNORE;
    
    public CopyAction()
    {
        super("jeditcom-copy-thread");
        parent = null;
        this.source = null;
        this.destDir = null;
        progressDialog.cancelButton.addActionListener(new CopyCanceledAction());
    }
    
    public CopyAction(Component parentComponent)
    {
        super("jeditcom-copy-thread");
        this.parent = parentComponent;
        this.source = null;
        this.destDir = null;
        progressDialog.cancelButton.addActionListener(new CopyCanceledAction());
    }
    
    public CopyAction(Component parentComp, File[] source, File destDir)
    {
        super("jeditcom-copy-thread");
        this.parent = parentComp;
        this.source = source;
        this.destDir = destDir;
        progressDialog.cancelButton.addActionListener(new CopyCanceledAction());
    }
    
    class CopyCanceledAction implements ActionListener
    {
        public void actionPerformed(ActionEvent e)
        {
            stop = true;
            progressDialog.dispose();
        }
    }
    
    class FileExistsPrompt extends JEditCommanderDialog implements
        ActionListener
    {
        private JButton[] buttons = {new JButton("Overwrite"), new JButton("Overwrite All"),
            new JButton("Skip"), new JButton("Cancel")};
        private JLabel label = new JLabel("");
        
        public FileExistsPrompt()
        {
            this.setModal(true);
            this.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
            this.getContentPane().setLayout(new BorderLayout());
            this.getContentPane().add(label, BorderLayout.CENTER);
            
            JPanel buttonPane = new JPanel(new GridLayout(1, 0));
            for (int i = 0; i < buttons.length; i++)
            {
                buttons[i].addActionListener(this);
                buttonPane.add(buttons[i]);
            }
            
            this.getContentPane().add(buttonPane, BorderLayout.SOUTH);
            GUIUtilities.loadGeometry(this, "jeditcom-copy-overwrite-dialog");
            this.pack();
            
            this.addWindowListener(new WindowAdapter()
            {
                public void windowClosing(WindowEvent e)
                {
                    GUIUtilities.saveGeometry(FileExistsPrompt.this, 
                        "jeditcom-copy-overwrite-dialog");
                }
            });
        }
        
        public void actionPerformed(ActionEvent e)
        {
            Object src = e.getSource();
            if (src.equals(buttons[0]))
                existsAction = CopyAction.COPY_OVERWRITE;
            else if (src.equals(buttons[1]))
                existsAction = CopyAction.COPY_OVERWRITE_ALL;
            else if (src.equals(buttons[2]))
                existsAction = CopyAction.COPY_SKIP;
            else 
                existsAction = CopyAction.COPY_STOP;
            
            this.dispose();
        }
        
        public void setLabelText(String text)
        {
            this.label.setText(text);
        }
    }
    
    public void run()
    {
        copy(source, destDir);
        this.yield();
    }
    
    public void setParent(Component parentComp)
    {
        this.parent = parentComp;
    }
    
    public void setSourceFile(File[] file)
    {
        this.source = file;
    }
    
    public void setDestinationDir(File dir)
    {
        this.destDir = dir;
    }
    
    public boolean copy(File[] source, File destDir)
    {
        long totalSize = 0;
        long readTotal = 0;
        for (int i = 0; i < source.length; i++)
        {
            if (source[i].isDirectory() == true)
                continue;
            
            totalSize += source[i].length();
        }
        
        progressDialog.setTotalLabelText("Total bytes to copy: " + 
            String.valueOf(totalSize));
        progressDialog.show();
        
        for (int i = 0; i < source.length; i++)
        {   
            if (source[i].isDirectory() == true)
                continue;
            
            progressDialog.setFileLabelText("Copying file " + source[i].getName());
            File destFile = new File(destDir.getPath() + System.getProperty("file.separator") +
                source[i].getName());
            Log.log(Log.DEBUG, null, "[JEDIT COMMANDER] Destination File: " +
                destFile.getPath());
        
            InputStream fileReader = null;
            try
            {    
                fileReader = new BufferedInputStream(new FileInputStream(source[i]));
            }
            catch (IOException ioex)
            {
                String[] args = {source[i].getPath()};
                GUIUtilities.error(parent, "jeditcom.copy.filenotfound", args);
                progressDialog.dispose();
                return false;
            }
        
            OutputStream fileWriter = null;
            try
            {
                if (destFile.exists() == true && 
                    existsAction != CopyAction.COPY_OVERWRITE_ALL)
                {
                    FileExistsPrompt feDialog = new FileExistsPrompt();
                    feDialog.setLabelText("File " + destFile.getPath() + 
                        " already exists. Please select an action below:");
                    feDialog.pack();
                    feDialog.show();
                    Log.log(Log.DEBUG, null, "File " + destFile.getPath() + 
                        " already exists");
                }
                
                if (existsAction == CopyAction.COPY_STOP)
                {
                    progressDialog.dispose();
                    fileReader.close();
                    return false;
                }
                else if (existsAction == CopyAction.COPY_SKIP)
                {
                    fileReader.close();
                    continue;
                }
                
                fileWriter = new FileOutputStream(destFile);
            }
            catch (Exception ex)
            {
                String[] args = {destFile.getPath()};
                GUIUtilities.error(parent, "jeditcom.copy.cannotcreate", args);
                progressDialog.dispose();
                return false;
            }
        
            int counter = 0;
            int readSoFar = 0;
            long bufLength = source[i].length();
        
            try
            {
                byte[] b = new byte[4096];
            
                while ((counter = fileReader.read(b, 0, b.length)) != -1)
                {
                    fileWriter.write(b, 0, counter);
                    readSoFar += counter;
                    readTotal += counter;
                    float fprogress = ((float) readSoFar / (float) bufLength);
                    int iprogress = (int) (fprogress * 100);
                    float totalFProgress = ((float) readTotal / (float) totalSize);
                    int totalIProgress = (int) (totalFProgress * 100);
                    
                    progressDialog.setTotalProgress(totalIProgress);                    
                    progressDialog.setFileProgress(iprogress);
                    
                    if (stop == true)
                    {
                        Log.log(Log.MESSAGE, null, "Copying canceled by a user");
                        fileReader.close();
                        fileWriter.close();
                        return false;
                    }
                }
            }
            catch (Exception ex)
            {
                String[] args = {source[i].getPath(), ex.getMessage()};
                GUIUtilities.error(parent, "jeditcom.copy.ioerror", args);
                ex.printStackTrace();
                progressDialog.dispose();
                return false;
            }
        
            try
            {
                fileReader.close();
                fileWriter.close();  
            }
            catch (Exception ex)
            {
                ex.printStackTrace();
                progressDialog.dispose();
                return false;
            }
        }
        
        progressDialog.dispose();
        
        return true;
    }
}