import java.util.*;
import org.gjt.sp.util.*;

class FileFiltering
{
    public static String prefix;
    public static String suffix;
    public static boolean filteringOn = false;
    public static boolean filteringEnabled = true;
    
    /**
     * This method creates a suffix and prefix from
     * parameter <code>text</code>. The text should
     * usually include * character as a wild card
     *
     * @return true if valid filter was created,
     * otherwise false
     */
public static boolean createFilter(String text)
    {
        // Check to see that text isn't empty
        if (text.length() < 1 || text == null)
            text = "*";
        
        /*First we check to see if the filtering is off -
        text only contains a * */
        text = text.trim();
        if (text.length() == 1)
        {
            if (text.equalsIgnoreCase("*"))
            {
                prefix = null;
                suffix = null;
                filteringOn = false;
                return false;
            }
        }
        
        // Now check if there is '*' character
        boolean hasWild = false;
        char wildChar = '*';
        char check;
        for (int i = 0; i < text.length(); i++)
        {
            check = text.charAt(i);
            if (check == wildChar)
            {
                hasWild = true;
                break;
            }
        }
        
        if (!hasWild)
        {
            filteringOn = true;
            prefix = text;
            suffix = null;
            return true;
        }
        
        // Now check to see if there is prefix
        boolean hasPrefix = false;
        check = text.charAt(0);
        if (check != wildChar)
        {
            hasPrefix = true;
        }
        
        // Suffix
        boolean hasSuffix = false;
        check = text.charAt(text.length() - 1);
        if (check != wildChar)
        {
            hasSuffix = true;
        }
        
        StringTokenizer st = new StringTokenizer(text, "*", false);
        int tokenCount = 0;
        int allTokens = st.countTokens();
        if (allTokens > 2)
        {
            Log.log(Log.DEBUG, null, "[jeditcom] Warning: file filter reports " +
                String.valueOf(allTokens) + " tokens");
            return false;
        }
        
        String token = "";
        while (st.hasMoreTokens() == true)
        {
            tokenCount++;
            token = st.nextToken();
            
            if (tokenCount == 1 && hasPrefix == true)
            {
                prefix = token;
                suffix = null;
            }
            else if (tokenCount == 1 && hasPrefix == false)
            {
                suffix = token;
                prefix = null;
            }
            else if (tokenCount == 2 && hasPrefix == true)
            {
                suffix = token;
            }
        }
        
        Log.log(Log.DEBUG, null, "[jeditcom] prefix: " + prefix);
        Log.log(Log.DEBUG, null, "[jeditcom] suffix: " + suffix);
        
        return true;
    }
    
    /**
     * Returns an array containig all files that match a criteria
     * @param files - an array of all files to be checked for matching
     * @param prefix - prefix to match to (can be null)
     * @param suffix - suffix to match to (can be null)
     * @return An array of all matching files
     */
    public static JEditCommanderFile[] getFilesMatchingFilter(JEditCommanderFile[]
        files, String prefix, String suffix)
    {
        Vector matchingFiles = new Vector();
        
        if (prefix != null && suffix == null)
        {
            for (int i = 0; i < files.length; i++)
            {
                if (FileFiltering.matchesPrefix(files[i].getFile().getName(), prefix) == true)
                    matchingFiles.addElement(files[i]);
            }
        }
        else if (prefix == null && suffix != null)
        {
            for (int i = 0; i < files.length; i++)
            {
                if (FileFiltering.matchesSuffix(files[i].getFile().getName(), suffix) == true)
                    matchingFiles.addElement(files[i]);
            }
        }
        else if (prefix != null && suffix != null)
        {
            boolean pm = false;
            boolean sm = false;
            for (int i = 0; i < files.length; i++)
            {
                pm = FileFiltering.matchesPrefix(files[i].getFile().getName(), prefix);
                sm = FileFiltering.matchesSuffix(files[i].getFile().getName(), suffix);
                
                if (pm == true && sm == true)
                {
                    matchingFiles.addElement(files[i]);
                }
            }
        }
        
        JEditCommanderFile[] matchingArray = new JEditCommanderFile[matchingFiles.size()];
        for (int i = 0; i < matchingArray.length; i++)
        {
            matchingArray[i] = (JEditCommanderFile) matchingFiles.elementAt(i);
        }
        
        return matchingArray;
    }
    
    private static boolean matchesPrefix(String fileName, String prefix)
    {
        return (fileName.startsWith(prefix));
    }
    
    private static boolean matchesSuffix(String fileName, String suffix)
    {
        return (fileName.endsWith(suffix));
    }
}