import javax.swing.*;
import java.awt.*;

public class JEditCommanderButtonPane extends JPanel
{
	private JButton[] commandButtons;
	private String[] buttonNames = {"Edit", "EditView", "Append", "Info",
          "Copy", "RenMov", "Mkdir", "Delete", "Quit"};
        private String[] toolTips = {"Opens selected files in jEdit",
          "Opens selected files in new View", 
          "Appends all of the selected files to a jEdit Buffer",
          "Displays info for selected files in a jEdit Buffer", 
          "Copies a set of files to a directory", 
          "Renames/moves the first selected file", "Creates a new directory in the current directory", 
          "Deletes selected files", "Closes jEdit Commander"};
	
	public JEditCommanderButtonPane()
	{
		setLayout(new GridLayout(1, 0));
		commandButtons = new JButton[buttonNames.length];
		
		initButtons();
	}
	
	/**
	 * Creates all the buttons, assigns them
	 * names and ActionListeners and adds them
	 * to the panel
	 */
	private void initButtons()
	{
		int alen = commandButtons.length;
		for (int i = 0; i < alen; i++)
		{
			commandButtons[i] = new JButton(buttonNames[i]);
			commandButtons[i].addActionListener(
				JEditCommanderActions.commanderActions[i]);
                        commandButtons[i].setToolTipText(toolTips[i]);
			add(commandButtons[i]);
		}
	}
	
	/**
	 * Returns the button at index index
	 * from the button array.
	 *
	 * @throws ArrayIndexOutOfBoundsException
	 * if index out of range
	 */
	public JButton getButton(int index) throws ArrayIndexOutOfBoundsException
	{
		if (index >= commandButtons.length)
			throw new ArrayIndexOutOfBoundsException(index);
		
		return commandButtons[index];	
	}
}