import javax.swing.*;
import javax.swing.table.*;
import java.awt.*;
import org.gjt.sp.util.*;

class JEditCommanderCellRenderer extends JTextField implements TableCellRenderer 
{
   	public JEditCommanderCellRenderer() 
	{
        super();
		setBorder(null);
    }

	/**
	 * This function is responsible for rendering a table cell which happens
	 * to be represented as a JTextField in our case. It changes the apperance
	 * of a JTextField and/or its text if, for example, a file represented in
	 * the current cell is actually a directory.
	 * Specified by the javax.swing.table.TableCellRenderer interface
	 */
	public Component getTableCellRendererComponent(JTable table, Object value, 
		boolean isSelected, boolean hasFocus, int row, int column) 
	{
		JEditCommanderTableModel model = (JEditCommanderTableModel) table.getModel();
		JEditCommanderFile[] files = model.getCurrentFiles();
		JEditCommanderFile theFile = files[row];
		
		resetAppearance();
		
		switch (column)
		{
			case 0:
				setText((String) value);
				if (getHorizontalAlignment() != JTextField.LEFT);
					setHorizontalAlignment(JTextField.LEFT);
				break;
			case 1:
				setText(value.toString());
				break;
			default:
				setText(value.toString());
				break;
		}
		
		if (theFile.isHidden == true)
		{
			if (getForeground() != Color.gray)
				setForeground(Color.gray);
		}

		
		if (theFile.isDir == true)
		{
			// Font
			Font currentFont = getFont();
			if (!currentFont.isBold())
			{
				Font dirFont = currentFont.deriveFont(Font.BOLD);
				setFont(dirFont);
			}
		}
		
		if (isSelected)
		{
			if (getBackground() != Color.red)
				setBackground(Color.red);
			
			if (getForeground() != Color.white)
				setForeground(Color.white);	
		}

		
		return this;
	}
	
	/**
	 * This is how a cell looks like if ALL
	 * of the following is FALSE:
	 * The cell is selected
	 * The cell contains directory
	 * The cell is marked
	 * The cell contains a hidden file
	 */
	private void resetAppearance()
	{
		// Font
		Font currentFont = getFont();
		if (!currentFont.isPlain())
		{
			Font defaultFont = currentFont.deriveFont(Font.PLAIN);
			setFont(defaultFont);
		}
		if (getForeground() != Color.black)
				setForeground(Color.black);
		
		// Alignment
		if (getHorizontalAlignment() != JTextField.RIGHT);
			setHorizontalAlignment(JTextField.RIGHT);
		
		// Selection
		if (getBackground() != Color.white)
			setBackground(Color.white);
		if (getForeground() != Color.black)
				setForeground(Color.black);	
	}
}