import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import org.gjt.sp.jedit.*;
import org.gjt.sp.util.*;
import java.io.*;

public class JEditCommanderGUI extends JFrame
{
	private JEditCommanderSplitPane splitPane;
	private JEditCommanderButtonPane buttonPane;
        
        public JEditCommanderGUI()
	{
		super(jEdit.getProperty("JEditCommander.frame.title"));

                Image icon = jEdit.getFirstView().getIconImage();
                this.setIconImage(icon);

		/*int fWidth = Integer.parseInt(
			jEdit.getProperty("JEditCommander.frame.width"));
		int fHeight = Integer.parseInt(
			jEdit.getProperty("JEditCommander.frame.height"));
		setSize(fWidth, fHeight);*/

		setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);

		getContentPane().setLayout(new BorderLayout());

		//Listeners
		addComponentListener(new JEditCommanderComponentListener());
		addWindowListener(new JEditCommanderFrameListener());
                addKeyListener(new KeyHandler());

		// Add button pane
		buttonPane = new JEditCommanderButtonPane();
		getContentPane().add(buttonPane, BorderLayout.SOUTH);

		// Add split pane
		splitPane = new JEditCommanderSplitPane();
		getContentPane().add(splitPane, BorderLayout.CENTER);

		// Now that we know about size and everything
                JEditCommanderSplitPane.leftDirField.setText(
			JEditCommanderPlugin.leftDirectory.getPath());

		JEditCommanderActions.changeDir.setTable(
			splitPane.getTable(JEditCommanderSplitPane.LEFT_TABLE));

                EditInBufferAction editBuffer = (EditInBufferAction)
                  JEditCommanderActions.commanderActions[0];
                editBuffer.setTable(
			splitPane.getTable(JEditCommanderSplitPane.LEFT_TABLE));

                EditInNewViewAction editView = (EditInNewViewAction)
                  JEditCommanderActions.commanderActions[1];
                editView.setTable(
			splitPane.getTable(JEditCommanderSplitPane.LEFT_TABLE));

                JEditCommanderPlugin.leftTable =
                  splitPane.getTable(JEditCommanderSplitPane.LEFT_TABLE);

                this.pack();
                GUIUtilities.loadGeometry(this, "jeditcom-frame");
		splitPane.setDividerLocation((int) (getWidth()*0.66f));

                splitPane.getTable(JEditCommanderSplitPane.LEFT_TABLE).getColumnModel(
                  ).getColumn(0).setPreferredWidth((int) (getMiddle() * 0.4f));

		setVisible(true);
	}

	/**
	 * Component listener to place divider of the
	 * split pane to the middle
	 */
	 class JEditCommanderComponentListener extends ComponentAdapter
	 {
	 	public void componentResized(ComponentEvent e)
		{
			splitPane.setDividerLocation((int) (getWidth()*0.66f));
		}
	 }

	 /**
	  * This WindowListener handles the close operation
	  */
	 class JEditCommanderFrameListener extends WindowAdapter
	 {
	 	public void windowClosing(WindowEvent e)
		{
			handleExit();
		}
	 }
        
        class KeyHandler extends KeyAdapter
        {   
            public void keyPressed(KeyEvent e)
            {
                //Log.log(Log.DEBUG, null, "[jeditcom] Key typed: " + String.valueOf(e.getKeyChar()));
                //int keyCode = e.getKeyCode();
             
                if (e.getKeyChar() == '*' && e.isControlDown() == true)
                {
                    Log.log(Log.DEBUG, null, "[jeditcom] SelectAll triggered");
                    JEditCommanderActions.selectAll.actionPerformed(new ActionEvent(JEditCommanderPlugin.leftTable, 
                        ActionEvent.ACTION_LAST+1, "Select All"));
                }
                else if (e.getKeyChar() == '+' && e.isControlDown() == true)
                {
                    Log.log(Log.DEBUG, null, "[jeditcom] SelectByFilter triggered");
                    JEditCommanderActions.selectFilter.actionPerformed(new ActionEvent(this, 
                        ActionEvent.ACTION_LAST+1, "Select By Filter"));
                }
                else if (e.getKeyChar() == '/' && e.isControlDown() == true)
                {
                    Log.log(Log.DEBUG, null, "[jeditcom] DeselectAll triggered");
                    JEditCommanderActions.deselectAll.actionPerformed(new ActionEvent(this, 
                        ActionEvent.ACTION_LAST+1, "Deselect All"));
                }
                else if (e.getKeyChar() == '-' && e.isControlDown() == true)
                {
                    Log.log(Log.DEBUG, null, "[jeditcom] DeselectByFilter triggered");
                    JEditCommanderActions.deselectFilter.actionPerformed(new ActionEvent(this, 
                        ActionEvent.ACTION_LAST+1, "Deselect By Filter"));
                }
                else if (e.getKeyCode() == KeyEvent.VK_I && e.getModifiers() ==
                    InputEvent.CTRL_MASK)
                {
                    Log.log(Log.DEBUG, null, "[jeditcom] InvertSelection triggered");
                    JEditCommanderActions.invert.actionPerformed(new ActionEvent(this, 
                        ActionEvent.ACTION_LAST+1, "Invert Selection"));
                }
                else if (e.getKeyCode() == KeyEvent.VK_BACK_SPACE && e.isControlDown() == true)
                {
                    Log.log(Log.DEBUG, null, "[jeditcom] DirectoryUp triggered");
                    JEditCommanderActions.changeDir.actionPerformed(new ActionEvent(this, 
                        ActionEvent.ACTION_LAST+1, "Directory Up"));
                }
                else if (e.getKeyCode() == KeyEvent.VK_R && e.getModifiers() ==
                    InputEvent.CTRL_MASK)
                {
                    Log.log(Log.DEBUG, null, "[jeditcom] ListRoots triggered");
                    JEditCommanderActions.toRoot.actionPerformed(new ActionEvent(this, 
                        ActionEvent.ACTION_LAST+1, "List Roots"));
                }
                else if (e.getKeyCode() == KeyEvent.VK_H && e.getModifiers() ==
                    InputEvent.CTRL_MASK)
                {
                    Log.log(Log.DEBUG, null, "[jeditcom] GoHome triggered");
                    JEditCommanderActions.goHome.actionPerformed(new ActionEvent(this, 
                        ActionEvent.ACTION_LAST+1, "Go To Home Directory"));
                }
            else if (e.getKeyCode() == KeyEvent.VK_F && e.getModifiers() ==
                    InputEvent.CTRL_MASK)
                {
                    Log.log(Log.DEBUG, null, "[jeditcom] Refresh triggered");
                    JEditCommanderActions.refresh.actionPerformed(new ActionEvent(this, 
                        ActionEvent.ACTION_LAST+1, "Refresh"));
                }
        } // function
        } // inner class

	/**
	 * Returns the horizontal center of the frame
	 * Used for placing the JSplitPane's divider
	 */
	public int getMiddle()
	{
		return (getWidth() / 2);
	}

	/**
	 * Invoked when a user requests that jEdit
	 * Commander should exit
	 */
	public void handleExit()
	{
		Log.log(Log.DEBUG, null, "jEdit Commander Exit Requested");
		dispose();
	}
}