import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import org.gjt.sp.jedit.*;
import org.gjt.sp.util.*;

public class JEditCommanderTable extends JTable
{
        public JEditCommanderTable()
	{
		String initialDir = jEdit.getProperty("jeditcom.initial.dir");
                if (initialDir.equalsIgnoreCase("null"))
                    jEdit.setProperty("jeditcom.initial.dir", System.getProperty("user.home"));
                
                JEditCommanderTableModel newModel = new JEditCommanderTableModel(jEdit.getProperty("jeditcom.initial.dir"));
                this.setModel(newModel);
                JEditCommanderPlugin.leftDirectory = new java.io.File(jEdit.getProperty("jeditcom.initial.dir"));

		initialise();
	}

	public JEditCommanderTable(String initialDir)
	{
		super(new JEditCommanderTableModel(initialDir));
                JEditCommanderPlugin.leftDirectory = new java.io.File(initialDir);

		initialise();
	}

	/**
	 * This class handles user clicks on a table,
	 * but not the header
	 */
	class ClickHandler extends MouseAdapter
	{
		Point clickPoint;
		int clickRow;

		public void mouseClicked(MouseEvent event)
		{
			if (!hasFocus())
                            requestFocus();
                        
                        clickPoint = event.getPoint();
			clickRow = rowAtPoint(clickPoint);

			if (event.getClickCount() > 1 && clickRow > -1 &&
				(event.getModifiers() & InputEvent.BUTTON1_MASK) ==
					InputEvent.BUTTON1_MASK)
			{
				JEditCommanderFile[] files = getTableModel().getCurrentFiles();
				JEditCommanderFile theFile = files[clickRow];

				if (theFile.getFile().isDirectory())
				{
					ChangeDirAction chDir = JEditCommanderActions.changeDir;
					chDir.setGoDown(false);

					try
					{
						chDir.setDirectory(theFile.getFile());
					}
					catch (java.io.IOException ioex)
					{
						ioex.printStackTrace();
					}
					chDir.actionPerformed(new ActionEvent(JEditCommanderTable.this,
						ActionEvent.ACTION_LAST + 1, null));
					chDir.setGoDown(true);
				}
			}
		} // mouseClicked

                public void mouseReleased(MouseEvent e)
                {
                  if (e.isPopupTrigger())
                  {
                    if (getSelectedRows().length < 1)
                    {
                      Toolkit.getDefaultToolkit().beep();
                      return;
                    }
                    JEditCommanderPopupMenu popup = new JEditCommanderPopupMenu();
                    popup.show(e.getComponent(), e.getX(), e.getY());
                  }
                } // mouseReleased
	} // inner class

	/**
	 * This class handles user click on table header.
	 * It calls various methods from JEditCommanderFileHandler
	 * to sort the files
	 */
	class HeaderClickHandler extends MouseAdapter
	{
		public void mouseClicked(MouseEvent e)
		{
			int clickCount = e.getClickCount();
			TableColumnModel columnModel = getColumnModel();
                        int viewColumn = columnModel.getColumnIndexAtX(e.getX());
                        int column = convertColumnIndexToModel(viewColumn);

			if (clickCount > 1 && column > -1 && (e.getModifiers() &
				InputEvent.BUTTON1_MASK) == InputEvent.BUTTON1_MASK)
			{
				switch (column)
				{
					case 0:
						JEditCommanderFileHandler.sortType = 0;
						break;
					case 1:
						JEditCommanderFileHandler.sortType = 2;
						break;
					case 2:
						JEditCommanderFileHandler.sortType = 3;
						break;
					case 3:
						JEditCommanderFileHandler.sortType = 4;
						break;
					default:
						JEditCommanderFileHandler.sortType = -1;
						break;
				} // switch
			} // if
			else if (clickCount > 1 && column == 0 && (e.getModifiers() &
				InputEvent.BUTTON3_MASK) == InputEvent.BUTTON3_MASK)
					JEditCommanderFileHandler.sortType = 1;
                        else if (clickCount > 0 && column > 0 && (e.getModifiers() &
                                InputEvent.BUTTON3_MASK) == InputEvent.BUTTON3_MASK)
                                  JEditCommanderFileHandler.sortType = -1;
			else
				return;

			JEditCommanderFileHandler.sort(getTableModel().getCurrentFiles());
			getTableModel().fireTableRowsDeleted(0,
                          getTableModel().getCurrentFiles().length - 1);
                        getTableModel().fireTableRowsInserted(0,
                          getTableModel().getCurrentFiles().length - 1);
		} // mouseClicked
	} // inner class

        class SelectionDetector implements ListSelectionListener
        {
            public void valueChanged(ListSelectionEvent e)
            {
                //Log.log(Log.DEBUG, null, "[JEDIT COMMANDER] List Selection Event triggered");
                String text = "";
                int[] selected = JEditCommanderTable.this.getSelectedRows();
                JEditCommanderTableModel model = getTableModel();
                JEditCommanderFile[] commanderFiles = model.getCurrentFiles();

                if (selected.length < 1)
                {
                    JEditCommanderSplitPane.theBox.setText("");
                    return;
                }
                else if (selected.length == 1)
                {
                    text = FileInfoAction.getFileInfo(commanderFiles[selected[0]].getFile());
                }
                else if (selected.length > 1)
                {
                    long totalLength = 0;
                    StringBuffer fileBuffer = new StringBuffer();
                    for (int i = 0; i < selected.length; i++)
                    {
                        totalLength += commanderFiles[selected[i]].getFileSize();
                        fileBuffer.append(commanderFiles[selected[i]].getFileName() +
                            "\n");
                    }
                    StringBuffer sb = new StringBuffer();
                    sb.append("Directory: " + 
                            commanderFiles[selected[0]].getFile().getParent() + "\n");
                    sb.append("Selected files: " + selected.length + "\n");
                    sb.append("Total size: " + totalLength + " bytes\n\n");
                    sb.append("List of selected files/directories:\n");
                    sb.append(fileBuffer.toString());

                    text = sb.toString();
                }

                JEditCommanderSplitPane.theBox.setText(text);
            }
        }

	private void initialise()
	{
		JEditCommanderCellRenderer renderer = new JEditCommanderCellRenderer();
		setDefaultRenderer(Object.class, renderer);
		getColumnModel().getColumn(1).setCellRenderer(renderer);

		setColumnSelectionAllowed(false);

		/*int frameHalf = Integer.parseInt(jEdit.getProperty(
			"JEditCommander.frame.width")) / 2;
		getColumnModel().getColumn(0).setPreferredWidth(frameHalf / 3);*/

		addMouseListener(new ClickHandler());

		// Header
		getTableHeader().addMouseListener(new HeaderClickHandler());

                this.getSelectionModel().addListSelectionListener(new SelectionDetector());
	}

	public JEditCommanderTableModel getTableModel()
	{
		return ((JEditCommanderTableModel) getModel());
	}
}