import java.awt.event.*;
import javax.swing.*;
import java.io.*;
import org.gjt.sp.util.*;
import java.util.*;

class RootSwitchBox extends JComboBox
{
    File[] directories;
    int neutralPos = 0;
    
    public RootSwitchBox()
    {
        File curFile = JEditCommanderPlugin.leftDirectory;
        if (curFile == null)
            curFile = new File(System.getProperty("user.home"));
        
        this.updateForDir(curFile.getPath());
        this.setSelectedIndex(-1);
        this.addItemListener(new SelectionChangeHandler());
    }
    
    class SelectionChangeHandler implements ItemListener
    {
        public void itemStateChanged(final java.awt.event.ItemEvent e)
        {
            int selIndex = getSelectedIndex();
            if (selIndex < 0 || selIndex == neutralPos || selIndex == 0)
                return;
                        
            File dirToChangeTo = directories[selIndex];
            
            try
            {
                JEditCommanderActions.changeDir.setDirectory(dirToChangeTo);
                JEditCommanderActions.changeDir.setGoDown(false);
                JEditCommanderActions.changeDir.actionPerformed(new ActionEvent(
                    RootSwitchBox.this, ActionEvent.ACTION_LAST+1, "Change Directory"));
                JEditCommanderActions.changeDir.setGoDown(true);
            }
            catch (IOException ioex)
            {
                Log.log(Log.ERROR, null, "[jeditcom] Cannot change to dir " +
                    directories[selIndex].getPath());
            }
        }
    }
    
    /**
     * Returns all files that will appear in the combo box
     */
    private File[] getFilesForBox(String currentDirPath)
    {
        File currentDirF = new File(currentDirPath);
        Vector filesVector = new Vector();
        
        // Info
        filesVector.addElement(new File("Directory to switch to:"));
        
        // We add root filesystems
        File[] allRoots = File.listRoots();
        neutralPos = allRoots.length + 1;
        Log.log(Log.DEBUG, null, "[jeditcom] neutral pos: " + String.valueOf(neutralPos));
        for (int i = 0; i < allRoots.length; i++)
            filesVector.addElement(allRoots[i]);
        
        // Now add neutral element
        filesVector.addElement(new File("-"));
        
        // Now we add all the parent directories
        File parent = currentDirF.getParentFile();
        while (parent != null)
        {
            Log.log(Log.DEBUG, null, "[jeditcom] parent file: " + parent.getPath());
            filesVector.addElement(parent);
            parent = parent.getParentFile();
        }
        
        // Transform it to File[]
        File[] comboFiles = new File[filesVector.size()];
        for (int i = 0; i < comboFiles.length; i++)
        {
            comboFiles[i] = (File) filesVector.elementAt(i);
        }
        
        return comboFiles;
    }
    
    /**
     * Updates the contents of the box according
     * to current directory
     */
    public void updateForDir(String dir)
    {
        this.removeAllItems();        
        directories = this.getFilesForBox(dir);
        
        for (int i = 0; i < directories.length; i++)
        {
            if (JEditCommanderFileHandler.isRoot(directories[i]) == true)
            {
                this.addItem(directories[i].getPath());
            }
            else
            {
                this.addItem(directories[i].getName());
            }
        }
        
        this.setSelectedIndex(neutralPos);
    }
}